@@ -25,7 +25,7 @@ total number of sequences that are needed and then sending just those and
 translated indices."""
 
 
-def apply_pairwise_rect(metric, seqs1, *args, seqs2=None, ncpus=1, use_numba=False, uniqify=True, alphabet=parasail_aa_alphabet, **kwargs):
+def apply_pairwise_rect(metric, seqs1, *args, seqs2=None, ncpus=1, use_numba=False, uniqify=True, reexpand=True, alphabet=parasail_aa_alphabet, **kwargs):
     """Calculate distance between pairs of sequences in seqs1
     with sequences in seqs2 using metric and kwargs provided to
     metric.
@@ -95,11 +95,14 @@ def apply_pairwise_rect(metric, seqs1, *args, seqs2=None, ncpus=1, use_numba=Fal
     else:
         useqs = useqs1
         if len(useqs) == 1:
-            """Only one unique sequence (this is only a problem when seqs2=None"""
+            """Only one unique sequence (this is only a problem when seqs2=None)"""
             urect = np.zeros((1, 1))
-            if translate1:
-                urect = urect[seqs1_uind, :][:, seqs1_uind]
-            return urect
+            if reexpand:
+                if translate1:
+                    urect = urect[seqs1_uind, :][:, seqs1_uind]
+                return urect
+            else:
+                return urect, seqs1_uind, seqs1_uind
         else:
             pw_indices = list(itertools.combinations(range(len(useqs)), 2))
 
@@ -110,7 +113,7 @@ def apply_pairwise_rect(metric, seqs1, *args, seqs2=None, ncpus=1, use_numba=Fal
     
     if not use_numba:
         dtype = type(metric(useqs[0], useqs[0], **kwargs))
-        if ncpus > 1:
+        if ncpus > 1 and len(useqs) > 10:
             try:
                 """Was not able to get joblib to provide any speedup over 1 CPU, though did not test thoroughly.
                 multiprocessing.Pool works OK and provides speedup over 1 CPU"""
@@ -136,7 +139,7 @@ def apply_pairwise_rect(metric, seqs1, *args, seqs2=None, ncpus=1, use_numba=Fal
     else:
         seqs_mat, seqs_L = seqs2mat(useqs, alphabet=alphabet)
 
-        if ncpus > 1:
+        if ncpus > 1 and len(useqs) > 10:
             """Now a list of the chunked [chunksz x 2] arrays"""
             chunked_indices = [np.array(i, dtype=np.int64) for i in chunked_indices]
             
@@ -163,25 +166,44 @@ def apply_pairwise_rect(metric, seqs1, *args, seqs2=None, ncpus=1, use_numba=Fal
             passing the "vector" metric. Speed is quite comparable"""
             urect = metric(pw_indices, seqs_mat, seqs_L, *args, **kwargs)
             # urect = nb_distance_vec(seqs_mat, seqs_L, pw_indices, metric, *numba_args)
-
-    if seqs2 is None:
-        urect = scipy.spatial.distance.squareform(urect, force='tomatrix')
-        if translate1:
-            urect = urect[seqs1_uind, :][:, seqs1_uind]
+    if reexpand:
+        if seqs2 is None:
+            urect = scipy.spatial.distance.squareform(urect, force='tomatrix')
+            if translate1:
+                urect = urect[seqs1_uind, :][:, seqs1_uind]
+        else:
+            urect = urect.reshape((len(useqs1), len(useqs2)))
+        
+            if translate1:
+                urect = urect[seqs1_uind, :]
+            if translate2:
+                urect = urect[:, seqs2_uind]
+        return urect
     else:
-        urect = urect.reshape((len(useqs1), len(useqs2)))
-    
-        if translate1:
-            urect = urect[seqs1_uind, :]
-        if translate2:
-            urect = urect[:, seqs2_uind]
-    return urect
+        if seqs2 is None:
+            urect = scipy.spatial.distance.squareform(urect, force='tomatrix')
+            if not translate1:
+                seqs1_uind = np.arange(urect.shape[0])
+                seqs2_uind = np.arange(urect.shape[0])
+            else:
+                seqs2_uind = seqs1_uind
+        else:
+            urect = urect.reshape((len(useqs1), len(useqs2)))
+        
+            if not translate1:
+                seqs1_uind = np.arange(urect.shape[0])
+            if not translate2:
+                seqs2_uind = np.arange(urect.shape[1])
+        """Return the unexpanded pw matrix and indices to expand along axis=0 and axis=1"""
+        return urect, seqs1_uind, seqs2_uind 
 
 def apply_pairwise_sparse(metric, seqs, pairs, *args, ncpus=1, use_numba=False, alphabet=parasail_aa_alphabet, **kwargs):
     """Calculate distance between pairs of sequences in seqs using metric and kwargs
     provided to metric. Will only compute distances specified in pairs of indices in pairs.
     Results could be used to create a sparse matrix of pairwise distances.
 
+    Disadvantage here is that there is no attempt to avoid redundant distance calculations.
+
     Will use multiprocessing Pool if ncpus > 1.
 
     Though written to be used for distance calculations,
@@ -302,7 +324,8 @@ def apply_running_rect(metric, seqs1, seqs2, radius, density_est, *args, ncpus=1
         translate1 = True
 
     # nb_running_x(query_i, seqs_mat, seqs_L, radius, density_est=0.05, *args)
-
+    """This is wasteful. Should create a seqsmat slice of only the rect that is needed
+    each time."""
     seqs_mat, seqs_L = seqs2mat(useqs1 + seqs2, alphabet=alphabet)
     n1 = len(useqs1)
 
